@@ -43,14 +43,15 @@ gl_list_branches <- function(project, verb = httr::GET, ...) {
 
 #' List, create and delete branches
 #' 
-#' @param branch_name name of branch to create/delete
+#' @param branch name of branch to create/delete
 #' @param ref ref name of origin for newly created branch
 #' @rdname branches
 #' @export
-gl_create_branch <- function(project, branch_name, ref = "master", verb = httr::POST, ...) {
+gl_create_branch <- function(project, branch, ref = "master", verb = httr::POST, ...) {
   gitlab(gl_proj_req(project, c("repository", "branches"), ...),
          verb = httr::POST,
-         branch_name = branch_name,
+         branch_name = branch, ## This is legacy for API v3 use and will be ignored by API v4
+         branch = branch,
          ref = ref,
          auto_format = FALSE,
          ...) %>%
@@ -61,8 +62,8 @@ gl_create_branch <- function(project, branch_name, ref = "master", verb = httr::
 #' 
 #' @rdname branches
 #' @export
-gl_delete_branch <- function(project, branch_name, verb = httr::POST, ...) {
-  gitlab(gl_proj_req(project, c("repository", "branches", branch_name), ...),
+gl_delete_branch <- function(project, branch, verb = httr::POST, ...) {
+  gitlab(gl_proj_req(project, c("repository", "branches", branch), ...),
          verb = httr::DELETE,
          auto_format = FALSE,
          ...) %>%
@@ -98,14 +99,16 @@ gl_list_files <- purrr::partial(gl_repository, req = "tree") ## should have a re
 #' For \code{gl_file_exists} dots are passed on to \code{\link{gl_list_files}} and gitlab API call
 #' @export
 #' @rdname gl_repository
-gl_file_exists <- function(project, file_path, ...) {
+gl_file_exists <- function(project, file_path, ref, ...) {
   
   project_missing <- missing(project)
   
-  list(...) %>%
+  list(ref = ref,
+       ref_name = ref, ## This is legacy for API v3 use and will be ignored by API v4
+       ...) %>%
     iff(dirname(file_path) != ".", c, path = dirname(file_path)) %>%
     iffn(project_missing, c, project = project) %>%
-    pipe_into("args", do.call, what = list_files) %>%
+    pipe_into("args", do.call, what = gl_list_files) %>%
     dplyr::filter(name == basename(file_path)) %>%
     { nrow(.) > 0 }
 }
@@ -209,7 +212,7 @@ gl_get_file <- function(project,
 #' @param project Project name or id
 #' @param file_path path where to store file in gl_repository
 #' @param content file content (text)
-#' @param branch_name name of branch where to append newly generated commit with new/updated file
+#' @param branch name of branch where to append newly generated commit with new/updated file
 #' @param commit_message Message to use for commit with new/updated file
 #' @param overwrite whether to overwrite files that already exist
 #' @param ... passed on to \code{\link{gitlab}}
@@ -225,22 +228,22 @@ gl_push_file <- function(project,
                          file_path,
                          content,
                          commit_message,
-                         branch_name = "master",
+                         branch = "master",
                          overwrite = TRUE,
                          ...) {
   
-  exists <- gl_file_exists(project = project, file_path, ref_name = branch_name, ...)
+  exists <- gl_file_exists(project = project, file_path, ref = branch, ...)
   if (!exists || overwrite) {
-    gitlab(req = gl_proj_req(project = project, c("repository", "files"), ...),
-           branch_name = branch_name,
-           file_path = file_path,
+    gitlab(req = gl_proj_req(project = project, c("repository", "files", file_path), ...),
+           branch_name = branch,  ## This is legacy for API v3 use and will be ignored by API v4
+           branch = branch,
            content = content,
            commit_message = commit_message,
            verb = if (exists) { httr::PUT } else { httr::POST },
            ...)
   } else {
     tibble::data_frame(file_path = character(0),
-                       branch_name = character(0))
+                       branch = character(0))
   }
 }
 
